/*----------------------------------------------------------------------------
 Copyright:      
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        22.05.2017
 Description:  

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.   
------------------------------------------------------------------------------*/

#include <avr/io.h>
#include <avr/interrupt.h>

#include "usart.h"
#include "irmp.h"
#include "irmpsystem.h"
#include "programme.h"
#include "hsv.h"

//############################################################################
void timer1_init (void)
{
	//compare value: 1/15000 of CPU frequency
	OCR1A   = (F_CPU / F_INTERRUPTS) - 1;  
	//switch CTC Mode on, set prescaler to 1                                
	TCCR1B  = (1 << WGM12) | (1 << CS10);  
	//OCIE1A: Interrupt by timer compare                                 
	TIMSK1  = 1 << OCIE1A;                                                  
}

//############################################################################
//Timer1 output compare A interrupt service routine, called every 1/15000 sec
ISR(TIMER1_COMPA_vect)                                                          
{
	(void) irmp_ISR();                                                        
}

//############################################################################
//Hauptprogramm
int main(void)
{	
	uint8_t v = 50;
	uint8_t s = 100;
	uint16_t h = 0;
	uint8_t prog = 0;
	
	IRMP_DATA   irmp_data;	
	//USART Init
	DDRD |= (1<<PD1);
	usart_init(9600);
	
	//initialize SPI
	spi_init();
	//initialize IRMP
	irmp_init();                                                            
    //initialize timer1
    timer1_init(); 
                                                 
	//enable interrupts
	sei ();                                                                
	
	usart_write("Start\r\n");
	
	while(1){
        if (irmp_get_data (&irmp_data)){
			//usart_write("IR GET:\r\n");
	        //usart_write("Protocol: %2x \r\n",irmp_data.protocol);     
			//usart_write("Address:  %4x \r\n",irmp_data.address); 
			usart_write("Command:  %4x \r\n",irmp_data.command); 
			//usart_write("Flags:    %2x \r\n",irmp_data.flags);  
			
			switch (irmp_data.command){
				case 0x43: //>
					h+=5;
					if(h > 360) h = 0;
					break;
				case 0x44: //<
					if(h == 0) h = 360;
					if(h > 0) h-=5;
					break;
					
				case 0x46: //^
					v+=5;
					if(v > 100) v = 100;
					break;
				case 0x15: //v
					if(v > 0) v-=5;
					break;	
					
				case 0x42: //#
					s+=5;
					if(s > 100) s = 100;
					break;
				case 0x4a: //*
					if(s > 0) s-=5;
					break;
					
				case 0x52: //0
					prog = 0;
					break;
				case 0x16: //1
					prog = 1;
					break;
				case 0x19: //2
					prog = 2;
					break;	
				case 0x0d: //3
					prog = 3;
					break;
				case 0x0c: //4
					prog = 4;
					break;
				case 0x18: //5
					prog = 5;
					break;
				case 0x5e: //6
					prog = 6;
					break;
				case 0x08: //7
					prog = 7;
					break;
				case 0x1c: //8
					prog = 8;
					break;
				case 0x5a: //9
					prog = 9;
					break;				
			}                                                 
        }
		
		//*********************************************
		if(prog == 0){ //eine Farbe
			fx_color(h,s,v);
		}
		
		//*********************************************
		if(prog == 1){ //Regenbogen
			fx_regenbogen (s, v);
		}
		
		//*********************************************
		if(prog == 2){ //Run 
			fx_run (h,s,v);
			_delay_ms(20);
		}
		
		//*********************************************
		if(prog == 3){ //Lauflicht
			fx_rider (h,s,v);
			_delay_ms(20);
		}
		
		//*********************************************
		if(prog == 4){ //Lauflicht
			fx_fire();
			_delay_ms(100);
		}
		
		
		update_strip();	
	}	
}

