/*----------------------------------------------------------------------------
 Copyright:      
 Author:         Radig Ulrich
 Remarks:        
 known Problems:
 Version:        
 Description:    
------------------------------------------------------------------------------*/

#include "programme.h"

unsigned short get_seed()
{
	unsigned short seed = 0;
	unsigned short *p = (unsigned short*) (RAMEND+1);
	extern unsigned short __heap_start;
	
	while (p >= &__heap_start + 1)
	seed ^= * (--p);
	
	return seed;
}

void spi_init (void)
{
	// configure pins MOSI, SCK as output
	SPI_DDR |= (1<<SPI_MOSI) | (1<<SPI_SCK) | (1<<SPI_SS);
	// pull SCK high
	SPI_PORT |= (1<<SPI_SCK);
	
	//SPI: enable, master, positive clock phase, msb first, SPI speed fosc/2
	SPCR = (1<<SPE) | (1<<MSTR) | (1<<SPR0);
	SPSR = (1<<SPI2X);
}

void spi_put(uint8_t value)
{
	SPDR = value;
	while( !(SPSR & (1<<SPIF)) ) ;
}

void store_led_data (uint16_t x, uint8_t red, uint8_t green, uint8_t blue)
{
	if(x >= MAX_LEDS) return;
	led_data[x].r = red;
	led_data[x].g = green;
	led_data[x].b = blue;
}

//############################################################################
void update_strip (void)
{
	//WS2801
	for(uint16_t a=0;a < (MAX_LEDS);a++)
	{
		spi_put(led_data[a].r);
		spi_put(led_data[a].g);
		spi_put(led_data[a].b);
	}
	
	//WS2812
	//cli();
	//ws2812_sendarray((uint8_t *)&led_data[0],(MAX_LEDS*3));		// Send array to WS2812 LED. The output pin is low after send.
	//sei();
	
	_delay_ms(1);
}

//############################################################################
void fx_regenbogen (uint8_t s, uint8_t v){
	
	static uint16_t count = 0;
	
	uint8_t r,g,b;
	uint16_t tmp = 0;
	
	for(uint16_t a=0;a < (MAX_LEDS);a++){
		tmp = count + a * (360 / MAX_LEDS);
		if (tmp > 360) tmp -= 360;
		
		hsv_to_rgb(tmp,s,v,&r,&g,&b);
		store_led_data (a,r,g,b);
	}
	count ++;
	if(count > 360) count = 0;
}

//############################################################################
void fx_color (uint16_t h,uint8_t s, uint8_t v){
	uint8_t r,g,b;
	
	hsv_to_rgb(h,s,v,&r,&g,&b);
	for(uint16_t a=0;a < (MAX_LEDS);a++){
		store_led_data (a,r,g,b);
	}
}

//############################################################################
void fx_rider (uint16_t h,uint8_t s, uint8_t v){
	
	static uint16_t count = 0;
	static uint16_t state = 0;
		
	uint8_t r,g,b;
	
	hsv_to_rgb(h,s,v,&r,&g,&b);
	for(uint16_t a=0;a < (MAX_LEDS);a++){
		if((a > count) && (a < (count + 10))){
			store_led_data (a,r,g,b);
			}else{
			store_led_data (a,0,0,0);
		}
	}
	
	if(state > 128){
		count++;
		if (count >= MAX_LEDS){
			state = 0;
		}
		
		}else{
		if (count == 0){
			state = 255;
			}else{
			count--;
		}
	}
}

//############################################################################
void fx_run (uint16_t h,uint8_t s, uint8_t v){
	
	static uint16_t count = 0;
	static uint16_t state = 0;
		
	uint8_t r,g,b;
	
	
	hsv_to_rgb(h,s,v,&r,&g,&b);
	for(uint16_t a=0;a < (MAX_LEDS);a++){
		if(a < count){
			if(state > 128){
				store_led_data (a,r,g,b);
				}else{
				store_led_data (a,0,0,0);
			}

			}else{
			if(state > 128){
				store_led_data (a,0,0,0);
				}else{
				store_led_data (a,r,g,b);
			}
		}

	}
				
	count++;
	if (count > MAX_LEDS){
		count = 0;
		if (state > 128)
		{
			state = 0;
			}else{
			state = 255;
		}
					
	}
}


//############################################################################
void fx_fire (void)
{
	uint8_t r,g,b,v;
	uint16_t h;

	srand(get_seed());
	
	for(uint16_t a=0;a < (MAX_LEDS);a++){
		h = (rand() %20); // Begrenzung von 0 - 30	
		v = (rand() %100);
		hsv_to_rgb(h+5,100,v,&r,&g,&b);
		store_led_data (a,r,g,b);
	}
}